import React, { useState, useEffect, useCallback, useMemo } from "react";
import { Col, Row, Tooltip, Tree } from "antd";
import { useModels, useModel, useAuth } from "../hooks";
import { emptyRequest } from "../constants/requests";
import Select from "./Select";

const ArbolPermisos = ({
  conPerfil = false,
  urlModulo,
  urlPerfil = '',
  permisosCargados = [],
  alMarcar
}) => {

  const { user } = useAuth();

  const [ape, setApe] = useState([]) // Arbol de permisos expandido
  const [idPerfil, setIdPerfil] = useState(null) // Estado de id parar perfil de permisos
  const [checkedList, setCheckedList] = useState([])
  const [listaPermisos, setListaPermisos] = useState([])
  const [arbolPermisos, setArbolPermisos] = useState([])
  const [perfilRequest, setPerfilRequest] = useState(emptyRequest) // Petición para perfil de permisos
  const [modulosRequest, setModulosRequest] = useState([]) // Estado petición módulos permisos

  const modulosParams = useMemo(() => ({
    name: urlModulo,
    limite: 100,
    expand: 'permisos'
  }), [urlModulo])

  // Parámetros de perfil de permisos
  const perfilesParams = useMemo(() => {
    if (conPerfil) {
      return {
        name: urlPerfil,
        id: idPerfil,
        expand: 'permisos',
      }
    }

    return emptyRequest
  }, [conPerfil, idPerfil, urlPerfil])

  const { models: modulos } = useModels(modulosRequest)

  // Perfil de permisos
  const { model: perfilPermiso } = useModel(perfilRequest)

  const ordenarLista = useCallback((a, b) => {
    const nombreA = a.nombre
      ? a.nombre.normalize('NFD')?.replace(/\p{Diacritic}/gu, '')
      : a.title.normalize('NFD')?.replace(/\p{Diacritic}/gu, '')
    const nombreB = b.nombre
      ? b.nombre.normalize('NFD')?.replace(/\p{Diacritic}/gu, '')
      : b.title.normalize('NFD')?.replace(/\p{Diacritic}/gu, '')

    if (nombreA < nombreB) {
      return -1
    }

    if (nombreA > nombreB) {
      return 1
    }

    return 0
  }, [])

  const onCheckedTree = useCallback(
    (checkedList, info) => {
      checkedList = checkedList.map((item) => {
        const val = item?.replace('p-', '')
        return val
      })

      let _checkedList = []
      for (let i = 0, l = listaPermisos.length; i < l; i++) {
        const _permiso = listaPermisos[i]
        if (checkedList.includes(_permiso['id'])) {
          _checkedList.push(_permiso['id'])
        }
      }

      setCheckedList(_checkedList)

      alMarcar && alMarcar(_checkedList);
    },
    [alMarcar, listaPermisos]
  )

  useEffect(() => {
    setModulosRequest(modulosParams)
    return () => setModulosRequest(emptyRequest)
  }, [modulosParams])

  /**
   * useEffect para hacer petición de un perfil
   * dependiendo de la selección
   */
  useEffect(() => {
    setPerfilRequest(perfilesParams)
    return () => setPerfilRequest({})
  }, [perfilesParams])


  useEffect(() => {
    if (modulos && modulos?.length) {
      let _arbolPermisos = []
      let _listaPermisos = []
      let _ape = []
      let _modulosPermisos = modulos

      _modulosPermisos.sort((a, b) => ordenarLista(a, b))

      for (let i = 0, l = _modulosPermisos.length; i < l; i++) {
        let _permisos = _modulosPermisos[i]['permisos']
        let _modulos = []
        for (let j = 0, k = _permisos?.length; j < k; j++) {
          let nPermiso = {
            title: `${_permisos[j]['nombre']} (${_permisos[j]['id']})`,
            descripcion: `${_permisos[j]['descripcion']}`,
            id: _permisos[j]['id'],
            selectable: false,
            key: `p-${_permisos[j]['id']}`,
          }
          _listaPermisos.push(nPermiso)
          _modulos.push(nPermiso)
        }

        _modulos.sort((a, b) => ordenarLista(a, b))

        const llave = `m-${modulos[i]['id']}`
        _ape.push(llave)
        if (modulos[i]?.id === 'ADMN' && user?.rol !== 'Super Administrador')
          continue
        _arbolPermisos.push({
          title: modulos[i]['nombre'],
          key: llave,
          id: modulos[i]['id'],
          children: _modulos,
        })
      }

      setApe(_ape)
      setListaPermisos(_listaPermisos)
      setArbolPermisos(_arbolPermisos)
    }
  }, [modulos, ordenarLista, user?.rol])

  useEffect(() => {
    if (conPerfil) {
      if (perfilPermiso) {
        onCheckedTree([])
        let permisos = []
        for (let i = 0, l = perfilPermiso?.permisos?.length; i < l; i++) {
          permisos.push(
            `m-${perfilPermiso?.permisos[i]?.idModulo}`,
            `p-${perfilPermiso?.permisos[i]?.id}`
          )
        }
        onCheckedTree(permisos)
      }
    }
  }, [conPerfil, onCheckedTree, perfilPermiso])

  useEffect(() => {
    setCheckedList(permisosCargados)
  }, [permisosCargados])

  return (
    <Row gutter={[10, 10]}>
      {conPerfil && (
        <Col span={24}>
          <Select
            modelsParams={{
              name: urlPerfil,
              limite: 20,
              ordenar: 'nombre-asc',
            }}
            labelProp="nombre"
            valueProp="id"
            placeholder="Seleccione un Perfíl de permisos"
            render={(_, row) => `${row?.clave} - ${row?.nombre}`}
            onSelect={(v) => setIdPerfil(v)}
          />
        </Col>
      )}
      <Col span={24}>
        <Tree
          checkable
          defaultExpandedKeys={ape}
          checkedKeys={checkedList?.map((i) => `p-${i}`)}
          onCheck={onCheckedTree}
          treeData={arbolPermisos}
          titleRender={(item) => {
            if (item?.descripcion !== null) {
              return (
                <Tooltip title={item?.descripcion}>
                  {item?.title}
                </Tooltip>
              )
            } else {
              return item?.title
            }
          }}
        />
      </Col>
    </Row>
  )
}

export default ArbolPermisos