import React, { useState } from "react";
import {
  ArrowRightOutlined,
  LockOutlined,
  UserOutlined,
  LoadingOutlined,
} from "@ant-design/icons";
import { Button, Form, Input, Spin, Typography, List, Row, Col } from "antd";
import { useAuth } from "../../hooks";
import { Link, useNavigate } from "react-router-dom";
import Recuperar from "./Recuperar";

const SignInStyles = {
  container: {
    background: "#fff",
    backdropFilter: "blur(50px)",
    boxShadow: "0 2px 10px 2px rgb(0 0 0 / 10%)",
    borderRadius: 6,
    padding: "16px 20px 20px 20px",
  },
  logoContainer: {
    textAlign: "center",
    marginBottom: 20,
    // hacer que se vea mas pequeño
    "& img": {
      width: 200,
    },
  },
};

const antIcon = (
  <LoadingOutlined
    style={{
      fontSize: 24,
    }}
    spin
  />
);

const reglas = {
  correo: [
    {
      type: "email",
    },
  ],
  clave: [
    {
      min: 6,
    },
  ],
};

const Ingresar = () => {
  const { signIn, sessionLoading } = useAuth();
  const navigate = useNavigate();
  const onFinish = (values) => {
    const { correo, clave } = values;
    signIn(correo, clave);
  };

  const data = ["Formatos", "Guía de uso", "Aviso de privacidad"];

  return (
    <Row gutter={[10, 10]}>
      {/* Listado Izquierdo */}
     {/*  <Col
        sm={24}
        md={6}
        style={{
          background: "#fff",
          backdropFilter: "blur(50px)",
          boxShadow: "0 2px 10px 2px rgb(0 0 0 / 10%)",
          borderRadius: 6,
        }}
      >
        <div style={{ padding: 20 }}>
          <Typography.Title level={5} style={{ margin: 0 }}>
            Documentación
          </Typography.Title>
          <List
            size="small"
            header={<div>Accesos</div>}
            bordered
            dataSource={data}
            renderItem={(item) => (
              <List.Item style={{ cursor: "pointer" }}>
                <ArrowRightOutlined style={{ color: "green" }} /> {item}
              </List.Item>
            )}
          />
          <p>
            <i>
              Si deseas solicitar información a otros gobiernos estatales,
              <a href="#" target="_blank" rel="noreferrer">
                {" "}
                da clic aquí.
              </a>
            </i>{" "}
          </p>
        </div>
      </Col> */}

      {/* Formulario */}
      <Col sm={24} md={12}  lg={24}>
        <div style={SignInStyles.container}>
          
          <div style={SignInStyles.logoContainer}>
            <img src={"https://solicitud.sagarhpa.com/img/EscudoSonora.png"} style={{ width: "60%" }} alt="" />
          </div>
          <Spin indicator={antIcon} spinning={sessionLoading}>
            <Form
              name="normal_login"
              className="login-form"
              layout="vertical"
              initialValues={{
                remember: false,
              }}
              onFinish={onFinish}
            >
              <Row gutter={[10, 10]}>
                <Col span={24}>
                  <Form.Item name="correo" label="Correo electrónico">
                    <Input
                      prefix={<UserOutlined className="site-form-item-icon" />}
                      placeholder="correo@ejemplo.com"
                    />
                  </Form.Item>
                </Col>
              </Row>

              <Row gutter={10}>
                <Col span={24}>
                  <Form.Item
                    name="clave"
                    label="Contraseña"
                    rules={reglas.clave}
                    extra={
                      <Button
                        type="link"
                        block
                        className="login-form-button"
                        style={{ padding: 0, textAlign: "right" }}
                        onClick={()=>navigate('/recuperar-contrasena')}
                      >
                        ¿Olvidaste tu contraseña?
                      </Button>
                    }
                  >
                    <Input.Password
                      prefix={<LockOutlined className="site-form-item-icon" />}
                      type="new-password"
                      autoComplete="off"
                      autoCorrect="off"
                      placeholder="Contraseña"
                    />
                  </Form.Item>
                </Col>
              </Row>

              <Row gutter={[10, 10]} justify="center">
                <Col span={8}>
                  <Form.Item>
                    <Button
                      type="primary"
                      htmlType="submit"
                      block
                      className="login-form-button"
                    >
                      Ingresar
                    </Button>
                  </Form.Item>
                </Col>
              </Row>
            </Form>
          </Spin>
        </div>
      </Col>
      <br />
    </Row>
  );
};

export default Ingresar;
