import { Form, Input, Button, Spin, Row, Col } from 'antd'
import { useEffect, useMemo, useState } from 'react'
import HttpService from '../../../services/httpService'
import { respuestas } from '../../../utilities'
import { useNavigate } from 'react-router-dom'
import { useQuery, useModel } from '../../../hooks'
import { commonRules } from '../../../constants/rules'
import DefaultLayout from '../../../components/layouts/DefaultLayout'


const endpoints = {
  estado: "estado",
};

const EstadoDetalle = () => {
  const [form] = Form.useForm()
  const navigate = useNavigate()
  const [loading, setLoading] = useState(false)
  const query = useQuery()
  const id = query.get("id")
  const [request, setRequest] = useState({})

  // const extraParams = useMemo(() => ({
  //   idAgenda: id,
  // }), [id])

  const requestParams = useMemo(() => ({
    name: endpoints.estado,
    id,
    // extraParams
  }), [id])


  const { model, modelLoading } = useModel(request)

  useEffect(() => {
    if (id) {
      setRequest(requestParams)
    }
    return () => {
      setRequest({})
    }
  }, [id, requestParams])

  useEffect(() => {
    if (model) {
      form.setFieldsValue({ 
        ...model,
      })
    }
  }, [form, model])

  const onFinish = async (values) => {
    try {
      setLoading(true);

      let body = {
        ...values,
      };

      if (id) {
        body.id = id
      }

      const res = await HttpService.post(`${endpoints.estado}/guardar`, body);
      respuestas(res);
      if (res?.status === 200) {
        navigate('/administracion/catalogos/estados')
      }
    } catch (error) {
      console.log(error);
      setLoading(false);
    } finally {
      setLoading(false);
    }
  }

  if (modelLoading) {
    return <Spin
      size="large"
      style={{ display: "block", margin: "auto", marginTop: "50px" }}
    />
  }


  return (
    <DefaultLayout>
      <Form
      layout="vertical"
      name="basic"
      form={form}
      onFinish={onFinish}
      onFinishFailed={() => { }}
    >
      <Row gutter={16}>
        <Col span={24}>
          <h2>Información del Estado</h2>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Nombre"
            name="nombre"
            rules={[
              commonRules.requerido,
            ]}
          >
            <Input />
          </Form.Item>
        </Col>
        <Col md={8} xs={12}>
          <Form.Item
            label="Abreviatura"
            name="abreviacion"
            rules={[
              commonRules.requerido,
            ]}
          >
            <Input />
          </Form.Item>
        </Col>
        <Col span={6}>
            <Form.Item label="ID Sagarhpa" name="idSagarhpa">
              <Input type="number"/>
            </Form.Item>
          </Col>
        <Col span={24}>
          <Form.Item>
            <Button
              type="primary"
              htmlType="submit"
              style={{ marginTop: "30px" }}
              loading={loading}
            >
              Guardar
            </Button>
          </Form.Item>
        </Col>
      </Row>
    </Form>
    </DefaultLayout>
  )
}

export default EstadoDetalle