import React, { useState } from 'react'
import { Input, Form, Row, Col, Button } from 'antd'
import HttpService from '../../../services/httpService'
import { respuestas } from '../../../utilities'
import { SaveOutlined } from '@ant-design/icons'
import { useNavigate } from 'react-router-dom'
import DefaultLayout from '../../../components/layouts/DefaultLayout'
import { commonRules } from '../../../constants/rules'


export const Formulario = ({ model = null, id, alTerminar, endPoint, url, idModelo, media }) => {
  const [form] = Form.useForm()
  const navigate = useNavigate()

  const [guardando, setGuardando] = useState(false)

  const handleFinish = async (values) => {

    try {
      setGuardando(true)

      let body = {
        ...values,
      }

      if (id) {
        body[idModelo] = id
      }

      const res = await HttpService.post(`${endPoint}/guardar`, body)
      if (respuestas(res)) {
        navigate(url)
      }
    } catch (e) {
      console.log(e)
    } finally {
      setGuardando(false)
    }

  }

  React.useEffect(() => {
    if (model || model !== null) {
      form.setFieldsValue({
        ...model,
      })
    } else {
      form.resetFields()
    }
  }, [form, model])

  return (
    <DefaultLayout>
      <Form
        layout="vertical"
        name="basic"
        form={form}
        onFinish={handleFinish}
        onFinishFailed={() => {}}
      >
        <Row gutter={[10, 10]}>
          <Col span={24}>
            <h2>Información de Nivel</h2>
          </Col>
          <Col span={24} md={12}>
            <Form.Item
              label="Clave"
              name="clave"
              rules={[commonRules.requerido]}
            >
              <Input size="large" />
            </Form.Item>
          </Col>
          <Col span={12}>
            <Form.Item label="Nombre" name="nombre">
              <Input/>
            </Form.Item>
          </Col>
          <Col span={24}>
            <Form.Item>
              <Button
                size="large"
                type="primary"
                htmlType="submit"
                loading={guardando}
                style={{ marginTop: "30px" }}
              >
                Guardar
              </Button>
            </Form.Item>
          </Col>
        </Row>
      </Form>
    </DefaultLayout>
  );

}
