import { useRef, useState, useCallback, useMemo } from "react";
import { Form, Tooltip } from "antd";
import { PlusOutlined } from "@ant-design/icons";
import { Tabla } from "../../components";
import { SimpleTableLayout } from "../../components/layouts";
import { ActionsButton } from "../../components";
import { isEllipsis, eliminarRegistro } from "../../utilities";
import { Link, useNavigate } from "react-router-dom";
import Formulario from "./Formulario";

const endPoint = "condicionante";
const endPointEliminar = "condicionante/eliminar";

const Condicionantes = () => {
  let tablaRef = useRef(null);
  const navigate = useNavigate();
  const [form] = Form.useForm();
  const [buscarParams, setBuscarParams] = useState({});

  const onFinish = useCallback((values) => {
    const { q } = values;
    const params = {
      q: q ?? "",
      padre: true,
    };
    setBuscarParams(params);    
  }, []);

  const botones = useMemo(() => [
    {
      onClick: () => navigate(`/condicionantes/agregar`),
      props: { disabled: false, type: "primary", block: false },
      text: "Nuevo",
      icon: <PlusOutlined />,
    },
  ], [navigate]);

  const linkText = (value, row, key) => (
    <Link
      to={`/condicionantes/editar?id=${row.id}`}
      style={{ color: "black" }}
    >
      {isEllipsis(columns, key) ? (
        <Tooltip title={value}>{value}</Tooltip>
      ) : (
        value
      )}
    </Link>
  );

  const columns = [
    {
      title: "Acciones",
      key: "id",
      dataIndex: "id",
      width: 100,
      align: "center",
      render: (_, item) => (
        <ActionsButton
          data={[
            {
              label: "Editar",
              onClick: () =>
                navigate(`/condicionantes/editar?id=${item?.id}`),
            },
            {
              label: "Eliminar",
              onClick: () => {
                eliminarRegistro(item?.titulo, item?.id, endPointEliminar, () =>
                  tablaRef?.current?.refresh()
                );
              },
              danger: true,
            },
          ]}
        />
      ),
    },
    {
      title: "Título",
      key: "titulo",
      dataIndex: "titulo",
      render: linkText,
    },
  ];

  return (
    <SimpleTableLayout
      btnGroup={{
        btnGroup: botones,
      }}
    >
      <Formulario
        form={form}
        onFinish={onFinish}
      />
      <Tabla
        columns={columns}
        nameURL={endPoint}
        extraParams={buscarParams}
        scroll={{ x: "30vw" }}
        order={'creado-desc'}
        ref={tablaRef}
      />
    </SimpleTableLayout>
  );
};

export default Condicionantes;
